import { Component, OnInit } from '@angular/core';
import { FileService } from 'src/app/services/file.service';
import { Router, ActivatedRoute } from '@angular/router';
import { MockService } from 'src/app/services/mock.service';
import { BPMNMock } from 'src/app/models/bpmn-mock';
import { instanceOfDelegateMock, instanceOfExternalServiceMock } from './mock-functions';
import { FlowService } from 'src/app/services/flow.service';
import { BPMNFlowElement } from 'src/app/models/bpmn-flow-element';
import { ExternalTopic } from 'src/app/models/external-topic';
import { ProjectService } from 'src/app/services/project.service';
import { SingleProjectConfig } from 'src/app/models/single-project-config';

@Component({
  selector: 'app-mocks',
  templateUrl: './mocks.component.html',
  styleUrls: ['./mocks.component.css']
})
export class MocksComponent implements OnInit {

  actualProject: number;
  actualProjectConfig: SingleProjectConfig;
  diagram: string;
  isDiagramLoading: boolean;
  delegateMocks: Array<BPMNMock>;
  externalServiceMocks: Array<BPMNMock>;
  activeMock: BPMNMock;
  isProjectLoading: boolean;
  isMocksLoading: boolean;
  isDeleteLoading: boolean;

  servicesToHighlight: Array<string>;

  delegateMockErrors: Array<String>;
  externalServiceMockErrors: Array<String>;
  
  externalTopics: Array<ExternalTopic>;
  delegateServiceTasks: Array<BPMNFlowElement>;
  
  constructor(private mockService: MockService, private projectService: ProjectService, private flowService: FlowService, private fileService: FileService, private route: ActivatedRoute) {
    this.route.params.subscribe(params =>
      this.actualProject = params['projectId']
    )
  }

  async ngOnInit() {
    await this.loadProject();
    await this.loadExternalTopics();
    await this.loadDelegateServiceTasks();
    await this.loadMocks();
    await this.loadBPMNDiagram();
  }

  async loadProject() {
    this.isProjectLoading = true;
    try {
      let result = await this.projectService.getProject(this.actualProject);
      this.actualProjectConfig = result;
    } finally {
      this.isProjectLoading = false;
    }
  }

  async loadBPMNDiagram() {
    this.isDiagramLoading = true;
    try {
      let result = await this.fileService.getBPMNDiagram(this.actualProject);
      this.diagram = result;
    } finally {
      this.isDiagramLoading = false;
    }
  }

  async loadExternalTopics() {
    try {
      let result = await this.flowService.getExternalTopics(this.actualProject);
      this.externalTopics = result;
    } finally {

    }
  }

  async loadDelegateServiceTasks() {
    try {
      let result = await this.flowService.getDelegateServiceTasks(this.actualProject);
      this.delegateServiceTasks = result;
    } finally {

    }
  }

  async loadMocks() {
    this.isMocksLoading = true;
    try {
      let result = await this.mockService.getMocks(this.actualProject);
      
      this.delegateMocks = result
        .filter(mock => instanceOfDelegateMock(mock));
      this.externalServiceMocks = result
        .filter(mock => instanceOfExternalServiceMock(mock));

      this.delegateMockErrors = [];
      this.externalServiceMockErrors = [];

      this.delegateMocks.forEach(() => this.delegateMockErrors.push(""));
      this.externalServiceMocks.forEach(() => this.externalServiceMockErrors.push(""));

    } finally {
      this.isMocksLoading = false;
    }
  }

  highlightService(serviceTask: string) {
    this.servicesToHighlight = [serviceTask];
    window.scrollTo(0, 0);
  }

  highlightServices(externalTopic: string) {
    let serviceTasks = this.externalTopics
      .find(topic => topic.name == externalTopic)
      .serviceTasks
      .map(serviceTask => serviceTask.id);

    this.servicesToHighlight = serviceTasks;
    window.scrollTo(0, 0);
  }

  async deleteDelegateMock(deleteMock: string, index: number) {
    this.isDeleteLoading = true;
    try {
      await this.mockService.deleteMock(this.actualProject, deleteMock);
      await this.loadMocks();
    } catch (err) {
      if (err.status === 409) {
        this.delegateMockErrors[index] = "Mock kann auf Grund bestehender Testfälle nicht gelöscht werden";
      } else {
        throw err;
      }
    } finally {
      this.isDeleteLoading = false;
    }
  }

  async deleteExternalServiceMock(deleteMock: string, index: number) {
    this.isDeleteLoading = true;
    try {
      await this.mockService.deleteMock(this.actualProject, deleteMock);
    } catch (err) {
      if (err.status === 409) {
        this.delegateMockErrors[index] = "Mock kann auf Grund bestehender Testfälle nicht gelöscht werden";
      } else {
        throw err;
      }
    } finally {
      await this.loadMocks();
      this.isDeleteLoading = false;
    }
  }
}
