import { Component, OnInit } from '@angular/core';
import { BPMNExternalServiceMock } from 'src/app/models/bpmn-external-service-mock';
import { FormGroup, FormBuilder, Validators, FormArray } from '@angular/forms';
import { MockService } from 'src/app/services/mock.service';
import { Router, ActivatedRoute } from '@angular/router';
import { BPMNFlowElement } from 'src/app/models/bpmn-flow-element';
import { FlowService } from 'src/app/services/flow.service';
import { VariableProposals } from 'src/app/models/variable-proposals';
import ValidationUtils from 'src/app/utils/validation';
import { ExternalTopic } from 'src/app/models/external-topic';

@Component({
  selector: 'app-new-external-mock',
  templateUrl: './new-external-mock.component.html',
  styleUrls: ['./new-external-mock.component.css']
})
export class NewExternalMockComponent implements OnInit {

  isMockLoading: boolean;
  mock: BPMNExternalServiceMock;
  actualProject: number;
  actualMock: string;
  externalTopics: Array<ExternalTopic>;
  variableInfos: Array<VariableProposals>;

  compareOperatorsString: Array<string> = ["==", "!="];
  compareOperatorsInt: Array<string> = ["==", "!=", "<", ">", "<=", ">="];
  compareOperatorsBool: Array<string> = ["==", "!="];

  dataTypes = new Map<string, string>();

  isEdit: boolean;

  isLoading: boolean;

  isError: boolean;

  public formGroup: FormGroup;

  constructor(private _fb: FormBuilder, private flowService: FlowService, private mockService: MockService, private router: Router, private route: ActivatedRoute) {
    this.route.params.subscribe(params => {
      this.actualProject = params['projectId'];
      this.actualMock = params['mockId'];
    })
  }

  async ngOnInit() {
    this.isLoading = true;

    await this.loadExternalTopics();
    await this.loadVariableInfo();
    this.initVariableDataTypes();

    if (this.actualMock) {
      this.isEdit = true;
      await this.loadMock();
    } else {
      this.isEdit = false;

      this.initNewMock();
    }

    this.initFormGroup();

    this.initVariableValues();
    this.formGroup.setValue(this.mock);

    this.isLoading = false;
  }

  async loadExternalTopics() {
    try {
      let result = await this.flowService.getExternalTopics(this.actualProject);
      this.externalTopics = result;
    } finally {

    }
  }

  async loadVariableInfo() {
    try {
      let result = await this.mockService.getVariableInfo(this.actualProject);
      this.variableInfos = result;
    } finally {

    }
  }

  async loadMock() {
    this.isMockLoading = true;
    try {
      let result = await this.mockService.getExternalServiceMock(this.actualProject, this.actualMock);
      this.mock = result;
    } finally {
      this.isMockLoading = false;
    }
  }

  async save() {
    this.isError = false;
    if (!this.formGroup.invalid) {

      let mockToSave = this.extractForm();
      console.log(mockToSave);

      if (this.isEdit) {
        await this.updateMock(mockToSave);
      } else {
        await this.saveMock(mockToSave);
      }
    } else {
      ValidationUtils.validateAllFormFields(this.formGroup);
      this.isError = true;
    }
  }

  async saveMock(mockToSave: BPMNExternalServiceMock) {
    try {
      await this.mockService.createExternalServiceMock(this.actualProject, mockToSave);
      this.router.navigate(['../'], { relativeTo: this.route });
    } catch (err) {
      if (err.status === 409) {
        this.formGroup.controls['name'].setErrors({ 'mockIdTaken': true });
      } else {
        throw err;
      }
    } finally {
      //
    }
  }

  async updateMock(mockToSave: BPMNExternalServiceMock) {
    try {
      await this.mockService.updateExternalServiceMock(this.actualProject, this.actualMock, mockToSave);
      this.router.navigate(['../../'], { relativeTo: this.route });
    } catch (err) {
      if (err.status === 409) {
        this.formGroup.controls['name'].setErrors({ 'mockIdTaken': true });
      } else {
        throw err;
      }
    } finally {
      //
    }
  }

  extractForm() : BPMNExternalServiceMock {
    let formValue = this.formGroup.value;

    formValue.mockRules.forEach(mockRule => {
      mockRule.comparisons.forEach(comp => {
        if (this.dataTypes.get(comp.variable) == 'java.lang.Boolean') {
          comp.value = JSON.parse(comp.value);
        }
      });
      mockRule.variables.forEach(variable => {
        if (this.dataTypes.get(variable.variable) == 'java.lang.Boolean') {
          variable.value = JSON.parse(variable.value);
        }
      });
    });

    return formValue;
  }

  initNewMock() {
    this.mock = {
      name: "",
      externalTopic: "",
      mockRules: []
    }
  }

  initVariableValues() {
    this.mock.mockRules.forEach(rule => {
      rule.comparisons.forEach(comp => {
        if (this.dataTypes.get(comp.variable) == 'java.lang.Boolean') {
          comp.value = comp.value.toString();
        }
      });
      rule.variables.forEach(variable => {
        if (this.dataTypes.get(variable.variable) == 'java.lang.Boolean') {
          variable.value = variable.value.toString();
        }
      });
    })
  }

  initFormGroup() {
    this.formGroup = this._fb.group({
      name: ['', Validators.compose([Validators.required])],
      externalTopic: ['', Validators.compose([Validators.required])],
      mockRules: this._fb.array([])
    });

    this.mock.mockRules.forEach((mockRule, index) => {
      this.addMockRuleToFormGroup();
      mockRule.comparisons.forEach(comparison => this.addComparisonToMockRuleFormGroup(index));
      mockRule.variables.forEach(variable => this.addVariableToMockRuleFormGroup(index));
    });
  }

  addMockRuleToFormGroup() {
    let formGroupMockRule = this._fb.group({
      comparisons: this._fb.array([]),
      variables: this._fb.array([])
    });

    this.mockRulesForm.push(formGroupMockRule);
  }

  addComparisonToMockRuleFormGroup(mockRuleIndex: number) {
    let formGroupComparison = this._fb.group({
      variable: ['', Validators.compose([Validators.required])],
      value: ['', Validators.compose([Validators.required])],
      compareOperator: ['', Validators.compose([Validators.required])]
    });

    this.comparisonForm(mockRuleIndex).push(formGroupComparison);
  }

  addVariableToMockRuleFormGroup(mockRuleIndex: number) {
    let formGroupVariable = this._fb.group({
      variable: ['', Validators.compose([Validators.required])],
      value: ['', Validators.compose([Validators.required])]
    });

    this.variableForm(mockRuleIndex).push(formGroupVariable);
  }

  addMockRuleToValue() {
    let mockRule = {
      comparisons: [],
      variables: []
    }

    this.mock.mockRules.push(mockRule);
  }

  addComparisonToValue(mockRuleIndex: number) {
    let comparison = {
      variable: "",
      value: "",
      compareOperator: ""
    }

    this.mock.mockRules[mockRuleIndex].comparisons.push(comparison);
  }

  addVariableValueToValue(mockRuleIndex: number) {
    let variableValue = {
      variable: "",
      value: ""
    }

    this.mock.mockRules[mockRuleIndex].variables.push(variableValue);
  }

  addMockRule() {
    this.addMockRuleToFormGroup();
    this.addMockRuleToValue();
  }

  deleteMockRule(mockRuleIndex: number) {
    this.mockRulesForm.removeAt(mockRuleIndex);
    this.mock.mockRules.splice(mockRuleIndex, 1);
  }

  addComparison(mockRuleIndex: number) {
    this.addComparisonToMockRuleFormGroup(mockRuleIndex);
    this.addComparisonToValue(mockRuleIndex);
  }

  deleteComparison(mockRuleIndex: number, comparisonIndex: number) {
    this.comparisonForm(mockRuleIndex).removeAt(comparisonIndex);
    this.mock.mockRules[mockRuleIndex].comparisons.splice(comparisonIndex, 1);
  }

  addVariableValue(mockRuleIndex: number) {
    this.addVariableToMockRuleFormGroup(mockRuleIndex);
    this.addVariableValueToValue(mockRuleIndex);
  }

  deleteVariableValue(mockRuleIndex: number, variableValueIndex: number) {
    this.variableForm(mockRuleIndex).removeAt(variableValueIndex);
    this.mock.mockRules[mockRuleIndex].variables.splice(variableValueIndex, 1);
  }

  onComparisonVariableChange(i: number, c: number) {
    let form = this.getComparisonForm(i, c);
    let variable = form.get('variable').value;

    form.get('compareOperator').setValue("");

    if (variable && this.dataTypes.get(variable) == 'java.lang.Boolean') {
      form.get('value').setValue("null");
    } else {
      form.get('value').setValue("");
    }
  }

  onVariablesVariableChange(i: number, v: number) {
    let form = this.getVariablesForm(i, v);
    let variable = form.get('variable').value;

    if (variable && this.dataTypes.get(variable) == 'java.lang.Boolean') {
      form.get('value').setValue("null");
    } else {
      form.get('value').setValue("");
    }
  }

  get mockRulesForm(): FormArray {
    return <FormArray>this.formGroup.get('mockRules');
  }

  comparisonForm(index: number): FormArray {
    return <FormArray>this.mockRulesForm.controls[index].get('comparisons');
  }

  variableForm(index: number): FormArray {
    return <FormArray>this.mockRulesForm.controls[index].get('variables');
  }

  getComparisonVariable(i: number, c: number) {
    return (<FormArray>(<FormArray>this.formGroup.get('mockRules'))
      .controls[i].get('comparisons')).controls[c].get('variable').value;
  }

  getComparisonForm(i: number, c: number) {
    return (<FormArray>(<FormArray>this.formGroup.get('mockRules'))
      .controls[i].get('comparisons')).controls[c];
  }

  getVariablesVariable(i: number, v: number) {
    return (<FormArray>(<FormArray>this.formGroup.get('mockRules'))
      .controls[i].get('variables')).controls[v].get('variable').value;
  }

  getVariablesForm(i: number, v: number) {
    return (<FormArray>(<FormArray>this.formGroup.get('mockRules'))
      .controls[i].get('variables')).controls[v];
  }

  getVariableProposals(variableName: string) {
    return this.variableInfos //
      .find(variable => variable.variable == variableName); //
  }

  private initVariableDataTypes() {
    for (let variableValue of this.variableInfos) {
      this.dataTypes.set(variableValue.variable, variableValue.type);
    }
  }
}
